from .get_balances import perform_request as get_balances
from .get_coin_value import perform_request as get_coin_value
from .get_deposit_address import perform_request as get_deposit_address
from .get_exchange_estimate import perform_request as get_exchange_estimate
from .get_exchange_rate import perform_request as get_exchange_rate
from .get_mining_mode import perform_request as get_mining_mode
from .get_mining_speed import perform_request as get_mining_speed
from .get_supported_coins import perform_request as get_supported_coins
from .get_supported_fiat import perform_request as get_supported_fiat
from .get_user_id import perform_request as get_user_id
from .manual_withdraw import perform_request as manual_withdraw
from .set_automatic_withdraw import perform_request as set_automatic_withdraw
from .set_mining_mode import perform_request as set_mining_mode
